import { createSignal, createEffect } from 'solid-js';
import { createStore } from 'solid-js/store';

var dispatcher = /*#__PURE__*/createSignal({
  type: '$$INIT'
}); //@ts-ignores

var action = dispatcher[0]; //@ts-ignores

var dispatch = dispatcher[1];

function createAction(type) {
  var fx = function fx(payload) {
    return payload ? dispatch({
      type: type,
      payload: payload
    }) : dispatch({
      type: type
    });
  };

  fx._$atype = type;
  return fx;
}

var __state = {};
/**
 * A function that accepts an initial state, an object full of reducer
 * functions, and a "state name", and return actions object
 *
 */

function createReducer(options) {
  //@ts-ignores
  var _createStore = createStore(options.initialState),
      state = _createStore[0],
      setState = _createStore[1];

  __state[options.name] = state;
  var reducers = options.reducers;
  options.reducers = {};
  var actions = Object.keys(reducers).reduce(function (obj, key) {
    var type = options.name + "_" + key; //@ts-ignores

    options.reducers[type] = reducers[key]; //obj[key] = (payload: any) => ({ type, payload });

    obj[key] = createAction(type);
    return obj;
  }, {});
  createEffect(function () {
    var ac = action();
    var fx = options.reducers[ac.type]; //@ts-ignores

    if (typeof fx === 'function') fx(setState, ac);
  });
  if (typeof actions.init === 'function') setTimeout(function () {
    return actions.init();
  });
  return {
    actions: actions,
    state: state
  };
}
function getState(stateName) {
  if (stateName) return __state[stateName];
  return __state;
}

function _debounce(func, timeout) {
  var _this = this;

  if (timeout === void 0) {
    timeout = 300;
  }

  var timer;
  return function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    clearTimeout(timer);
    timer = setTimeout(function () {
      //@ts-ignore
      func.apply(_this, args);
    }, timeout);
  };
}

function subscribeEffect(actions, callback) {
  createEffect(function () {
    var ac = action();
    if (actions.includes(ac.type)) callback(ac);
  });
}

function subscribeEffectForSelection(actions, callback) {
  createEffect(function () {
    var ac = action();

    if (actions.includes(ac.type)) {
      callback(ac);
    }
  });
}

function on() {
  for (var _len2 = arguments.length, actions = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    actions[_key2] = arguments[_key2];
  }

  var _actions = actions.map(function (actionFn) {
    return actionFn._$atype;
  });

  return {
    debounce: function debounce(milliseconds) {
      return {
        effect: function effect(handlerFn) {
          return subscribeEffect(_actions, _debounce(handlerFn, milliseconds));
        },
        select: function select(handlerFn, init) {
          var arr = createSignal({
            loading: false,
            data: init,
            msg: null
          });
          var dataFn = syncData(arr[1]);
          subscribeEffectForSelection(_actions, _debounce(function (action) {
            return handlerFn(action, dataFn);
          }, milliseconds));
          return arr[0];
        }
      };
    },
    effect: function effect(handlerFn) {
      return subscribeEffect(_actions, handlerFn);
    },
    select: function select(handlerFn, init) {
      var arr = createSignal({
        loading: false,
        data: init,
        msg: null
      });
      var dataFn = syncData(arr[1]);
      subscribeEffectForSelection(_actions, function (action) {
        return handlerFn(action, dataFn);
      });
      return arr[0];
    }
  };
}

function syncData(state) {
  return {
    loading: function loading() {
      state(function (s) {
        return {
          data: s.data,
          msg: null,
          loading: true
        };
      });
    },
    success: function success(data) {
      state(function () {
        return {
          msg: null,
          data: data,
          loading: false
        };
      });
    },
    error: function error(msg) {
      state(function (s) {
        return {
          loading: s.loading,
          data: s.data,
          msg: msg
        };
      });
    }
  };
}

export { action, createAction, createReducer, dispatch, getState, on };
//# sourceMappingURL=solid-redux.esm.js.map
