import { SetStoreFunction } from 'solid-js/store';
export interface Action<T = any> {
    type: T;
}
export interface AnyAction extends Action {
    [extraProps: string]: any;
}
export declare type Reducer<S = any, A extends Action = AnyAction> = (setState: SetStoreFunction<S>, action: A) => void;
export declare type SelectHandler<T = any> = (action: PayloadAction<any>, data: {
    loading: () => void;
    success: (data: T) => void;
    error: (msg: any) => void;
}) => void;
export declare type ReducerMethods<State> = {
    [K: string]: Reducer<State, PayloadAction<any>>;
};
export declare type ValidateReducers<S, ACR extends ReducerMethods<S>> = ACR;
export declare type PayloadAction<P = void, T extends string = string> = {
    payload: P;
    type: T;
};
export interface ActionCreatorWithoutPayload<T extends string = string> {
    (): PayloadAction<void, T>;
}
export interface ActionCreatorWithPayload<P, T extends string = string> {
    (payload: P): PayloadAction<P, T>;
}
declare type ActionCreatorForReducer<R> = R extends (state: any, action: infer Action) => any ? Action extends {
    payload: infer P;
} ? ActionCreatorWithPayload<P> : ActionCreatorWithoutPayload : ActionCreatorWithoutPayload;
export declare type ReducerActions<Reducers extends ReducerMethods<any>> = {
    [Type in keyof Reducers]: ActionCreatorForReducer<Reducers[Type]>;
};
export interface ReducerOptions<State = any, R extends ReducerMethods<State> = ReducerMethods<State>, Name extends string = string> {
    name: Name;
    initialState: State;
    reducers: ValidateReducers<State, R>;
}
export declare type CreateReducer<State extends {}, R extends ReducerMethods<State> = ReducerMethods<State>> = {
    actions: ReducerActions<R>;
    state: State;
};
export declare type SelectState<T> = {
    loading: boolean;
    data: T;
    msg: any;
};
export declare type ActionFn<T = any> = ActionCreatorWithPayload<T> | ActionCreatorWithoutPayload;
export declare type EffectHandler<A = any> = (action: PayloadAction<A>) => void;
export declare type EffectHandlers = {
    [K: string]: EffectHandler<PayloadAction<any>>;
};
export declare type ValidateHandlers<ACR extends EffectHandlers> = ACR & {
    [T in keyof ACR]: ACR[T] extends {
        handler(dispatch: (action: AnyAction) => void, getState: () => any, action?: infer A): void;
    } ? {
        prepare(...a: never[]): Omit<A, 'type'>;
    } : {};
};
export {};
